try:
    from django.urls import URLPattern

except ImportError:  # Django < 2.0
    from django.core.urlresolvers import RegexURLPattern as URLPattern


from sitegate.toolbox import get_sitegate_urls


def test_get_sitegate_urls():
    urls = get_sitegate_urls()
    assert isinstance(urls, list)
    assert len(urls) == 1
    assert isinstance(urls[0], URLPattern)



import pytest

try:
    from django.urls import reverse

except ImportError:  # Django < 2.0
    from django.core.urlresolvers import reverse

from sitegate.signin_flows.modern import ModernSigninForm
from sitegate.signup_flows.modern import ModernSignupForm, InvitationSignupForm


URL_REGISTER = reverse('register')
URL_LOGIN = reverse('login')


@pytest.fixture
def user_signup(request_client):

    def user_signup_(user, params=None):
        default_data = {'email': user.email, 'password1': user.password_plain, 'signup_flow': 'ModernSignup'}
        default_data.update(params or {})
        return request_client().post(URL_REGISTER, default_data)

    return user_signup_


@pytest.fixture
def user_signin(request_client):

    def user_signin_(user, params=None, client=None):
        default_data = {'username': user.email, 'password': user.password_plain, 'signin_flow': 'ModernSignin'}
        default_data.update(params or {})
        if not client:
            client = request_client()
        return client.post(URL_LOGIN, default_data)

    return user_signin_


def _regenerate_email(self, name_len=20):
    self._email = '{}@mail.com'.format('a' * name_len)
    return self._email


def test_modern_signin(user_signin, user_signup, user_create, user_model):

    user = user_create(attributes={'email': 'user@host.com'})
    user.delete()

    # login with wrong credentials
    response = user_signin(user, {'username': 'nobody'})
    assert response.status_code == 200
    assert b'Please enter a correct username and password' in response.content

    # create deactivated test user
    response = user_signup(user)
    assert response['location'].endswith(reverse('ok'))

    # inactive user login
    response = user_signin(user)
    assert (
            b'This account is inactive.' in response.content or
            b'Please enter a correct username and password' in response.content)

    # activate user and try to login again
    assert user_model.objects.count() == 1
    user_model.objects.all().update(is_active=True)
    response = user_signin(user)
    assert response['location'].endswith(reverse('ok'))

    # how about to login when user is already signed in?
    response = user_signin(user, client=response.client)
    assert response['location'].endswith(reverse('fail'))

    response.client.logout()

    # more than one user with this e-mail
    user_create(attributes={'email': user.email})
    # user_model.objects.create(username='dummy', )
    response = user_signin(user)

    assert b'There is more than one user with this e-mail.' in response.content


def test_modern_signup(user_signin, user_signup, user_create, user_model):

    user = user_create(attributes={'email': 'user@host.com'})
    user.delete()

    # create deactivated user
    response = user_signup(user)
    assert response['location'].endswith(reverse('ok'))

    # auto_signin = False
    assert not str(response.client.cookies).strip()

    # he can't login yet
    assert not response.client.login(username=user.username, password=user.password_plain)

    # check fields
    db_user = user_model._default_manager.get(username=user.email)
    assert db_user.email == user.email
    assert db_user.username == user.email

    # try to sign up with the same data for a second time
    response = user_signup(user)
    assert response.status_code == 200
    assert b'A user with that e-mail already exists.' in response.content

    # try to signup with email too long
    user.email = '%s@some.com' % ('long' * 100)
    response = user_signup(user)

    assert response.status_code == 200
    assert b'length should be no more than' in response.content


class TestModernSignupForms(object):

    def test_modern_signup_attrs(self):
        f = ModernSignupForm()
        fields = set(f.fields.keys())

        assert 'username' not in fields
        assert 'email' in fields
        assert 'password1' in fields
        assert 'password2' not in fields

    def test_invitation_signup_attrs(self):
        f = InvitationSignupForm()
        fields = set(f.fields.keys())

        assert 'username' not in fields
        assert 'code' in fields
        assert 'email' in fields
        assert 'password1' in fields
        assert 'password2' not in fields


class TestModernSigninForms(object):

    def test_modern_signin_attrs(self):
        f = ModernSigninForm()
        fields = set(f.fields.keys())

        assert 'username' in fields
        assert 'password' in fields



import pytest


from sitegate.models import EmailConfirmation, BlacklistedDomain
from sitegate.signup_flows.base import SignupFlow
from sitegate.signup_flows.modern import ModernSignup

try:
    from django.urls import reverse

except ImportError:  # Django < 2.0
    from django.core.urlresolvers import reverse


def test_verify_email(user, request_client, messages):
    result = request_client().get(reverse('verify_email', args=['42']))
    assert result._headers['location'][1].endswith('/')
    assert len(messages) == 1
    assert 'Unable to verify' in messages

    result = request_client().get(reverse('verify_email', args=[EmailConfirmation.add(user)]))
    assert 'successfully verified' in messages
    assert len(messages) == 2


def test_is_blacklisted():

    BlacklistedDomain.objects.bulk_create([
        BlacklistedDomain(domain='denied1.com', enabled=False),
        BlacklistedDomain(domain='denied2.com', enabled=True),
        BlacklistedDomain(domain='denied3.com', enabled=True),
        BlacklistedDomain(domain='denied4.co.uk', enabled=True)
    ])

    assert not BlacklistedDomain.is_blacklisted('example1@denied1.com')
    assert BlacklistedDomain.is_blacklisted('example2@denied2.com')
    assert BlacklistedDomain.is_blacklisted('example3@some.denied3.com')
    assert BlacklistedDomain.is_blacklisted('example3@some.denied3.COM')
    assert not BlacklistedDomain.is_blacklisted('example4@co.uk')
    assert BlacklistedDomain.is_blacklisted('example4@denied4.co.UK')
    assert BlacklistedDomain.is_blacklisted('example4@some.Denied4.co.UK')


class TestGenericSignupFlow(object):

    def test_init_exception(self):

        with pytest.raises(NotImplementedError):
            SignupFlow()

    def test_getflow_name(self):
        assert ModernSignup.get_flow_name() == 'ModernSignup'



from sitegate.signup_flows.classic import ClassicWithEmailSignup, ClassicWithEmailSignupForm, ClassicSignupForm, \
    SimpleClassicSignupForm, SimpleClassicWithEmailSignupForm
from sitegate.utils import get_username_field


def test_signup(request_get, messages):
    flow = ClassicWithEmailSignup()
    assert flow.validate_email_domain
    assert not flow.verify_email
    assert not hasattr(flow, 'schedule_email')

    flow = ClassicWithEmailSignup(verify_email=True)
    assert flow.flow_args['verify_email']
    assert not flow.flow_args['activate_user']
    assert not flow.flow_args['auto_signin']
    assert hasattr(flow, 'schedule_email')

    form = ClassicWithEmailSignupForm({
        'username': 'abcom',
        'email': 'a@b.com',
        'password1': 'qwerty',
        'password2': 'qwerty',
    })
    form.flow = flow

    def fake_schedule_email(text, user, title):
        assert user.username == 'abcom'
        assert title == 'Account activation'

    flow.schedule_email = fake_schedule_email

    new_user = flow.add_user(request_get('/'), form)
    assert new_user.username == 'abcom'


class TestClassicSignupForms(object):

    def test_classic_signup_attrs(self):
        f = ClassicSignupForm()
        fields = set(f.fields.keys())

        assert get_username_field() in fields
        assert 'password1' in fields
        assert 'password2' in fields

    def test_classic_signup_validation(self, user_model):
        user_model._default_manager.create(
            username='test_duplicate', email='test_duplicate@mail.some')

        f = ClassicSignupForm(data={
            'username': 'test_duplicate',
            'email': 'test_duplicate@mail.some',
        })
        f.full_clean()
        assert 'username' in f.errors
        assert len(f.errors) == 3

        f = ClassicSignupForm(data={
            'username': 'test_duplicate',
            'email': 'test_duplicate@mail.some',
            'password1': 'password',
            'password2': 'password',
        })
        f.full_clean()
        assert 'username' in f.errors
        assert len(f.errors) == 1

    def test_simple_classic_signup_attrs(self):
        f = SimpleClassicSignupForm()
        fields = set(f.fields.keys())

        assert get_username_field() in fields
        assert 'password1' in fields
        assert 'password2' not in fields

    def test_classic_with_email_signup_attrs(self):
        f = ClassicWithEmailSignupForm()
        fields = set(f.fields.keys())

        assert get_username_field() in fields
        assert 'email' in fields
        assert 'password1' in fields
        assert 'password2' in fields

    def test_simple_classic_with_email_signup_attrs(self):
        f = SimpleClassicWithEmailSignupForm()
        fields = set(f.fields.keys())

        assert get_username_field() in fields
        assert 'email' in fields
        assert 'password1' in fields
        assert 'password2' not in fields


# @unittest.skipIf(settings.AUTH_USER_MODEL == 'auth.User', 'Custom user model required')
# class ClassicSignupCustomUserFormsTest(TestCase):
# 
#     def test_classic_signup_attrs():
#         f = ClassicSignupForm()
#         fields = set(f.fields.keys())
# 
#         assert 'email' in fields
#         assert 'password1' in fields
#         assert 'password2' in fields
# 
#     def test_classic_signup_validation():
#         get_user_model()._default_manager.create(
#             email='test_duplicate@mail.some',
#             date_of_birth=timezone.now().date(),
#         )
# 
#         f = ClassicSignupForm(data={
#             'email': 'test_duplicate@mail.some',
#         })
#         f.full_clean()
#         assert 'email' in f.errors
#         assert len(f.errors) == 3
# 
#         f = ClassicSignupForm(data={
#             'email': 'test_duplicate@mail.some',
#             'password1': 'password',
#             'password2': 'password',
#         })
#         f.full_clean()
#         assert 'email' in f.errors
#         assert len(f.errors) == 1
# 
#     def test_simple_classic_signup_attrs():
#         f = SimpleClassicSignupForm()
#         fields = set(f.fields.keys())
# 
#         assert 'username' not in fields
#         assert 'email' in fields
#         assert 'password1' in fields
#         assert 'password2' not in fields
# 
#     def test_classic_with_email_signup_attrs():
#         f = ClassicWithEmailSignupForm()
#         fields = set(f.fields.keys())
# 
#         assert 'username' not in fields
#         assert 'email' in fields
#         assert 'password1' in fields
#         assert 'password2' in fields
# 
#     def test_simple_classic_with_email_signup_attrs():
#         f = SimpleClassicWithEmailSignupForm()
#         fields = set(f.fields.keys())
# 
#         assert 'username' not in fields
#         assert 'email' in fields
#         assert 'password1' in fields
#         assert 'password2' not in fields



from django.conf.urls import url
from pytest_djangoapp.compat import get_urlpatterns

from sitegate.toolbox import get_sitegate_urls
from .views import login, register, HttpResponse


urlpatterns = get_urlpatterns([
    url(r'entrance/', lambda r: None),
    url(r'^ok/$', lambda r: HttpResponse('ok'), name='ok'),
    url(r'^fail/$', lambda r: HttpResponse('fail'), name='fail'),
    url(r'^login/$', login, name='login'),
    url(r'^register/$', register, name='register'),
] + get_sitegate_urls())



from sitegate.models import InvitationCode


def test_generate_code(user):
    assert InvitationCode.generate_code()


def test_add(user):
    code = InvitationCode.add(user)
    assert code
    assert code.creator == user
    assert code.acceptor is None
    assert code.time_created
    assert code.time_accepted is None
    assert not code.expired


def test_accept(user):
    code = InvitationCode.add(user)
    rows = InvitationCode.accept(code.code, user)
    assert rows == 1
    updated_code = InvitationCode.objects.get(code=code.code)
    assert updated_code.acceptor == user
    assert updated_code.time_accepted
    assert updated_code.expired



from django.http import HttpResponseRedirect

from sitegate.decorators import signin_view, signup_view, redirect_signedin, sitegate_view
from sitegate.signin_flows.classic import ClassicSignin
from sitegate.signin_flows.modern import ModernSignin
from sitegate.signup_flows.classic import ClassicSignup
from sitegate.signup_flows.modern import ModernSignup


def test_signin(request_get):
    # simple decoration
    response = signin_view(lambda req: req)(request_get('/signin/'))
    assert hasattr(response, 'sitegate')
    assert 'signin_forms' in response.sitegate
    assert 'signup_forms' not in response.sitegate

    # simple decoration + view with args
    response = signin_view(lambda req, a1, a2='b': (req, a1, a2))(request_get('/signin/'), 10, a2=20)
    assert hasattr(response[0], 'sitegate')
    assert 'signin_forms' in response[0].sitegate
    assert 'signup_forms' not in response[0].sitegate
    assert response[1] == 10
    assert response[2] == 20

    # stacking
    signin_view1 = signin_view(flow=ClassicSignin)
    signin_view2 = signin_view(flow=ModernSignin)
    response = signin_view1(signin_view2(lambda req: req))(request_get('/signin/'))
    assert hasattr(response, 'sitegate')
    assert 'signin_forms' in response.sitegate
    assert 'signup_forms' not in response.sitegate
    assert len(response.sitegate['signin_forms']) == 2
    dict_keys = list(response.sitegate['signin_forms'].keys())
    assert isinstance(response.sitegate['signin_forms'][dict_keys[0]], ClassicSignin.form)
    assert isinstance(response.sitegate['signin_forms'][dict_keys[1]], ModernSignin.form)

    # stacking with args
    signin_view1 = signin_view(flow=ClassicSignin, widget_attrs={'attr_name1': 'attr_value1'})
    signin_view2 = signin_view(flow=ModernSignin, widget_attrs={'attr_name2': 'attr_value2'})
    response = signin_view1(signin_view2(lambda req: req))(request_get('/signin/'))
    assert hasattr(response, 'sitegate')
    field = response.sitegate['signin_forms']['ClassicSignin'].fields.get('password')
    assert 'attr_name1' in field.widget.attrs
    field = response.sitegate['signin_forms']['ModernSignin'].fields.get('password')
    assert 'attr_name2' in field.widget.attrs


def test_signup(request_get):
    # simple decoration
    response = signup_view(lambda req: req)(request_get('/signup/'))
    assert hasattr(response, 'sitegate')
    assert 'signup_forms' in response.sitegate
    assert 'signin_forms' not in response.sitegate

    # simple decoration + view with args
    response = signup_view(lambda req, a1, a2='b': (req, a1, a2))(request_get('/signup/'), 10, a2=20)
    assert hasattr(response[0], 'sitegate')
    assert 'signup_forms' in response[0].sitegate
    assert 'signin_forms' not in response[0].sitegate
    assert response[1] == 10
    assert response[2] == 20

    # stacking
    signup_view1 = signup_view(flow=ClassicSignup)
    signup_view2 = signup_view(flow=ModernSignup)
    response = signup_view1(signup_view2(lambda req: req))(request_get('/signup/'))
    assert hasattr(response, 'sitegate')
    assert 'signup_forms' in response.sitegate
    assert 'signin_forms' not in response.sitegate
    assert len(response.sitegate['signup_forms']) == 2
    dict_keys = list(response.sitegate['signup_forms'].keys())
    assert isinstance(response.sitegate['signup_forms'][dict_keys[0]], ClassicSignup.form)
    assert isinstance(response.sitegate['signup_forms'][dict_keys[1]], ModernSignup.form)

    # stacking with args
    signup_view1 = signup_view(flow=ClassicSignup, widget_attrs={'attr_name1': 'attr_value1'})
    signup_view2 = signup_view(flow=ModernSignup, widget_attrs={'attr_name2': 'attr_value2'})
    response = signup_view1(signup_view2(lambda req: req))(request_get('/signup/'))
    assert hasattr(response, 'sitegate')
    field = response.sitegate['signup_forms']['ClassicSignup'].fields.get('password1')
    assert 'attr_name1' in field.widget.attrs
    field = response.sitegate['signup_forms']['ModernSignup'].fields.get('password1')
    assert 'attr_name2' in field.widget.attrs


def test_signinup(request_get):
    # simple decoration
    response = signin_view(signup_view(lambda req: req))(request_get('/entrance/'))
    assert hasattr(response, 'sitegate')
    assert 'signup_forms' in response.sitegate
    assert 'signin_forms' in response.sitegate

    # simple decoration + view with args
    response = signin_view(signup_view(lambda req, a1, a2='b': (req, a1, a2)))(
        request_get('/entrance/'), 10, a2=20)

    assert hasattr(response[0], 'sitegate')
    assert 'signup_forms' in response[0].sitegate
    assert 'signin_forms' in response[0].sitegate
    assert response[1] == 10
    assert response[2] == 20

    # stacking with args
    view_signin = signin_view(widget_attrs={'attr_name1': 'attr_value1'})
    view_signup = signup_view(widget_attrs={'attr_name2': 'attr_value2'})
    response = view_signin(view_signup(lambda req, a1, a2='b': (req, a1, a2)))(
        request_get('/entrance/'), 10, a2=20)

    assert hasattr(response[0], 'sitegate')
    assert 'signup_forms' in response[0].sitegate
    assert 'signin_forms' in response[0].sitegate
    assert response[1] == 10
    assert response[2] == 20
    field = response[0].sitegate['signin_forms']['ModernSignin'].fields.get('password')
    assert 'attr_name1' in field.widget.attrs
    field = response[0].sitegate['signup_forms']['ModernSignup'].fields.get('password1')
    assert 'attr_name2' in field.widget.attrs


def test_redirect(request_get, user_create):
    # logged in
    # default redirect
    request = request_get('/entrance/', user=user_create())
    response = redirect_signedin(lambda req: req)(request)
    assert isinstance(response, HttpResponseRedirect)

    # not logged in
    request = request_get('/entrance/', user=user_create(anonymous=True))
    response = redirect_signedin(lambda req: req)(request)
    assert not isinstance(response, HttpResponseRedirect)

    # paametrized decoration
    request = request_get('/entrance/', user=user_create())
    redirector = redirect_signedin('/abc/')
    response = redirector(lambda req: req)(request)
    assert isinstance(response, HttpResponseRedirect)
    assert response['Location'] == '/abc/'


def test_sitegate(request_get, user_create):
    # logged in
    request = request_get('/entrance/', user=user_create())
    response = sitegate_view(lambda req: req)(request)
    assert isinstance(response, HttpResponseRedirect)

    # not logged in

    # simple decoration
    request = request_get('/entrance/', user=user_create(anonymous=True))
    response = sitegate_view(lambda req: req)(request)
    assert not isinstance(response, HttpResponseRedirect)
    assert hasattr(response, 'sitegate')
    assert 'signup_forms' in response.sitegate
    assert 'signin_forms' in response.sitegate

    # simple decoration + view with args
    request = request_get('/entrance/', user=user_create(anonymous=True))
    response = sitegate_view(lambda req, a1, a2='b': (req, a1, a2))(request, 10, a2=20)
    assert not isinstance(response, HttpResponseRedirect)
    assert hasattr(response[0], 'sitegate')
    assert 'signup_forms' in response[0].sitegate
    assert 'signin_forms' in response[0].sitegate
    assert response[1] == 10
    assert response[2] == 20

    # parametrized decoration
    request = request_get('/entrance/', user=user_create(anonymous=True))
    view_sitegate = sitegate_view(widget_attrs={'attr_name1': 'attr_value1'})
    response = view_sitegate(lambda req, a1, a2='b': (req, a1, a2))(request, 10, a2=20)
    assert hasattr(response[0], 'sitegate')
    field = response[0].sitegate['signin_forms']['ModernSignin'].fields.get('password')
    assert 'attr_name1' in field.widget.attrs
    field = response[0].sitegate['signup_forms']['ModernSignup'].fields.get('password1')
    assert 'attr_name1' in field.widget.attrs
    assert response[1] == 10
    assert response[2] == 20



try:
    from django.http.response import HttpResponse
except ImportError:
    from django.http import HttpResponse
from django.template.base import Template
from django.template.context import Context

from sitegate.decorators import signup_view, redirect_signedin, signin_view


def response_from_string(request, string):
    return HttpResponse(Template(string).render(Context({'request': request})))


@signup_view(activate_user=False, auto_signin=False, validate_email_domain=False, redirect_to='ok')
def register(request):
    return response_from_string(request, '{% load sitegate %}{% sitegate_signup_form %}')


@redirect_signedin('fail')
@signin_view(redirect_to='ok')
def login(request):
    return response_from_string(request, '{% load sitegate %}{% sitegate_signin_form %}')
